#include <Wire.h>
#include <LiquidCrystal_I2C.h>
// Set the LCD address to 0x27 for a 16 chars and 2 line display
LiquidCrystal_I2C lcd(0x27, 16, 2);
// Rotary Encoder Inputs
#define inputCLK 10
#define inputDT 1
#define inputSW 0 // switch button
int counter = 0; 
int currentStateCLK;
int previousStateCLK; 
unsigned long previousMillis = 0; 
const long interval = 1000; 
int setTemperature = 0; // Variable to hold the set temperature
void setup() { 
  // Set encoder pins as inputs  
  pinMode (inputCLK,INPUT);
  pinMode (inputDT,INPUT);
  pinMode (inputSW,INPUT_PULLUP); // set the switch button as input
  lcd.init();
  
  // Setup Serial Monitor
  lcd.print("T: ");
  lcd.backlight();

  // Initialize I2C communication as Master
  Wire.begin();
} 

void loop() { 
  // Read the current state of inputCLK
  currentStateCLK = digitalRead(inputCLK);

  // If the previous and the current state of the inputCLK are different then a pulse has occurred
  if (currentStateCLK != previousStateCLK){ 

    // If the inputDT state is different than the inputCLK state then the encoder is rotating counterclockwise
    if ( digitalRead(inputDT) != currentStateCLK ) { 
      // Add this line to prevent the counter from going below zero
        counter ++; 
      
    } else {
      // Encoder is rotating clockwise
      if(counter>0){
      counter --;
      }
    }
    Serial.print("Position: ");
    Serial.println(counter);
    // Update the LCD with the new temperature value
    lcd.setCursor(2, 0); // Set the cursor on the third column and first row.
    lcd.print(counter);
    lcd.print(" C   "); // Print the temperature and clear any extra characters
  } 
  // Remember the previous state of the inputCLK state
  previousStateCLK = currentStateCLK; 
  // Check if the button is pressed
  if (digitalRead(inputSW) == LOW) {
    // Button is pressed - set the temperature
    setTemperature = counter;
    Serial.print("Temperature set to: ");
    Serial.println(setTemperature);
    lcd.setCursor(7, 0);
    lcd.print("SetT: ");
    lcd.print(setTemperature);
    lcd.print(" C   "); // Print the set temperature and clear any extra characters
    //Send the set temperature to the secondary device
    Wire.beginTransmission(0x8); // transmit to device with address 0x08
    Wire.write(setTemperature); // sends the set temperature
    Wire.endTransmission(); // stop transmitting
  }
    unsigned long currentMillis = millis();
  if (currentMillis - previousMillis >= interval) {
    previousMillis = currentMillis;
    Wire.requestFrom(0x8,1); // request 1 byte from secondary device with address 0x08
  }  
  if (Wire.available()) { // if there's a byte to read 
    int currentTemperature=1;
    currentTemperature = Wire.read(); // read the byte
    lcd.setCursor(1, 1); 
    lcd.print("CurTemp:");  
    lcd.setCursor(11,1);
    lcd.print(currentTemperature);
    lcd.print(" C   "); // Print the current temperature and clear any extra characters
    
 }
}
