#include <Arduino.h>
#include <max6675.h>
#include <Wire.h>
#include <LiquidCrystal_I2C.h>  // Include LiquidCrystal_I2C library
#define MAX6675_SO 9
#define MAX6675_CS 2
#define MAX6675_SCK 10
#define I2C_Secondary_ADDRESS 0x8
#define DIR_PIN 3
#define STEP_PIN 1
#define EN_PIN 0
unsigned long previousMillis = 0;
const long interval = 500;
int desiredTemp = 80;       // Desired hot end temperature in Celsius
const int maxPower = 1023;  // Maximum power output 
int heaterPin = 8;
#define STEPS 200 
MAX6675 thermocouple(MAX6675_SCK, MAX6675_CS, MAX6675_SO);
float Kp = 5;             // Proportional gain
float Ki = 0.15;          // Integral gain
float Kd = 5;             // Derivative gain (initially zero)
float previousError = 0;  // For integral term calculation
float totalError = 0;     // For integral term calculation
int currentTemp;
void setup() {
  Serial.begin(9600);
  pinMode(DIR_PIN, OUTPUT);
  pinMode(STEP_PIN, OUTPUT);
  pinMode(EN_PIN, OUTPUT);
  digitalWrite(EN_PIN, HIGH);
  pinMode(heaterPin, OUTPUT);
  Wire.begin(I2C_Secondary_ADDRESS);
  Wire.onReceive(receiveEvent);
  Wire.onRequest(requestEvent);  
}

void loop() {
  float error=6;
  unsigned long currentMillis = millis();
  if (currentMillis - previousMillis >= interval) {
    previousMillis = currentMillis;
    currentTemp = thermocouple.readCelsius();
    error = desiredTemp - currentTemp;
    // PID calculations
    float proportionalTerm = Kp * error;
    totalError += error;  // Integrate error over time
    totalError = constrain(totalError, 0, 100);
    float integralTerm = Ki * totalError;
    float derivativeTerm = Kd * (error - previousError);  // Calculate rate of change
    previousError = error;                                // Update for next iteration
    float powerOutput = proportionalTerm + integralTerm + derivativeTerm;
    powerOutput = map(powerOutput, 0, 50, 0, maxPower);  // Adjust power based on temperature difference
    // Apply safety limits
    powerOutput = constrain(powerOutput, 0, maxPower);
    powerOutput = 255 - powerOutput;
    // Control heater based on power output (PWM or analog)
    //digitalWrite(heaterPin, 1);
    analogWrite(heaterPin, powerOutput);
    //Serial.print("error");
    //Serial.println(error);

  }
  // Replace "heaterPin" with your actual heater control pin
  if (error >= -5 && error <= 5) {
      digitalWrite(EN_PIN, LOW);
      digitalWrite(DIR_PIN, HIGH);
  for(int i = 0; i < STEPS; i++) {
    digitalWrite(STEP_PIN, HIGH);
    delayMicroseconds(8000); // this delay needed for letting the motor step
    digitalWrite(STEP_PIN, LOW);
    delayMicroseconds(8000);
  }
  }
  
}
void receiveEvent(int numBytes) {
  byte recivedData = 0;
  if (Wire.available()) {
    recivedData = Wire.read();
    desiredTemp = recivedData;
  }
}
void requestEvent() {
  Wire.write(currentTemp);  // send actual temperature to primary board
}
